"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const slate_1 = require("slate");
const tokenizer_1 = require("../utils/tokenizer");
function isDisabled(char, options) {
    const disabled = options.disabled || {};
    if ((char === "*" || char === "_") && (disabled.strong || disabled.em)) {
        return true;
    }
    else if (char === "~" && disabled.del) {
        return true;
    }
    else if (char === "`" && disabled.codespan) {
        return true;
    }
    else {
        return false;
    }
}
function withMarkdown(editor, options = {}) {
    const { insertText, deleteBackward } = editor;
    // Text Completion: Automatically inserts a second char (*_`~)
    // and places the cursor between them.
    editor.insertText = (text) => {
        const { selection } = editor;
        if (selection &&
            slate_1.Range.isCollapsed(selection) &&
            !isDisabled(text, options)) {
            const [start] = slate_1.Range.edges(selection);
            const before = slate_1.Editor.before(editor, start, { unit: "line" });
            const beforeRange = before && slate_1.Editor.range(editor, before, start);
            const beforeText = beforeRange
                ? slate_1.Editor.string(editor, beforeRange, { voids: true })
                : "";
            const after = slate_1.Editor.after(editor, start);
            const afterRange = slate_1.Editor.range(editor, start, after);
            const afterChar = slate_1.Editor.string(editor, afterRange);
            if ((/[_`~]/.test(text) && /(^| |[_`~])$/.test(beforeText)) ||
                (text === "*" && (beforeText.endsWith(" ") || afterChar === "*"))) {
                slate_1.Transforms.insertText(editor, text);
                slate_1.Transforms.move(editor, { reverse: true });
            }
        }
        insertText(text);
    };
    // Automatically deletes markdown chars
    editor.deleteBackward = (unit) => {
        var _a;
        const { selection } = editor;
        if (selection && slate_1.Range.isCollapsed(selection)) {
            const [start] = slate_1.Range.edges(selection);
            const before = slate_1.Editor.before(editor, start);
            const beforeRange = slate_1.Editor.range(editor, start, before);
            const beforeChar = slate_1.Editor.string(editor, beforeRange);
            const beforeLine = slate_1.Editor.before(editor, start, { unit: "line" });
            const beforeLineRange = beforeLine && slate_1.Editor.range(editor, beforeLine, start);
            const beforeText = beforeLineRange && slate_1.Editor.string(editor, beforeLineRange);
            const after = slate_1.Editor.after(editor, start);
            const afterRange = slate_1.Editor.range(editor, start, after);
            const afterChar = slate_1.Editor.string(editor, afterRange);
            const removeSpan = ["*", "_", "`", "~"].includes(beforeChar) && beforeChar == afterChar;
            const listMatch = beforeText && beforeText.match(tokenizer_1.rules.listItemStart);
            const removeListSymbol = listMatch && listMatch[0] === beforeText;
            if (removeSpan && !isDisabled(beforeChar, options)) {
                slate_1.Transforms.delete(editor, { at: afterRange });
            }
            else if (removeListSymbol && !((_a = options.disabled) === null || _a === void 0 ? void 0 : _a.list)) {
                slate_1.Transforms.delete(editor, { at: beforeLineRange });
                slate_1.Transforms.insertNodes(editor, {
                    type: "paragraph",
                    children: [{ text: "" }],
                });
            }
        }
        deleteBackward(unit);
    };
    return editor;
}
exports.default = withMarkdown;
